@@ -63,6 +63,7 @@ from multiprocessing.process import BaseProcess
 
 from scalene.adaptive import Adaptive
 from scalene.runningstats import RunningStats
+from scalene.syntaxline import SyntaxLine
 from scalene import sparkline
 
 Filename = NewType("Filename", str)
@@ -363,6 +364,11 @@ class Scalene:
         Filename, Dict[LineNumber, Dict[ByteCodeIndex, float]]
     ] = defaultdict(lambda: defaultdict(lambda: defaultdict(float)))
 
+    # number of times samples were added for the above
+    __memory_malloc_count: Dict[
+        Filename, Dict[LineNumber, Dict[ByteCodeIndex, int]]
+    ] = defaultdict(lambda: defaultdict(lambda: defaultdict(int)))
+
     # mallocs attributable to Python, for each location in the program
     __memory_python_samples: Dict[
         Filename, Dict[LineNumber, Dict[ByteCodeIndex, float]]
@@ -383,6 +389,11 @@ class Scalene:
         lambda: defaultdict(int)
     )
 
+    # leak score tracking
+    __leak_score: Dict[Filename, Dict[LineNumber, float]] = defaultdict(
+        lambda: defaultdict(float)
+    )
+
     # how many CPU samples have been collected
     __total_cpu_samples: float = 0.0
 
@@ -1013,6 +1024,7 @@ class Scalene:
         # Iterate through the array to compute the new current footprint.
         # and update the global __memory_footprint_samples.
         before = Scalene.__current_footprint
+        prevmax = Scalene.__max_footprint
         for item in arr:
             _alloc_time, action, count, python_fraction = item
             count /= 1024 * 1024
@@ -1025,7 +1037,6 @@ class Scalene:
                 Scalene.__current_footprint -= count
             Scalene.__memory_footprint_samples.add(Scalene.__current_footprint)
         after = Scalene.__current_footprint
-
         # Now update the memory footprint for every running frame.
         # This is a pain, since we don't know to whom to attribute memory,
         # so we may overcount.
@@ -1064,6 +1075,7 @@ class Scalene:
                     python_frac / allocs
                 ) * (after - before)
                 Scalene.__malloc_samples[fname] += 1
+                Scalene.__memory_malloc_count[fname][lineno][bytei] += 1
                 Scalene.__total_memory_malloc_samples += after - before
             else:
                 Scalene.__memory_free_samples[fname][lineno][bytei] += (
@@ -1071,6 +1083,11 @@ class Scalene:
                 )
                 Scalene.__memory_free_count[fname][lineno][bytei] += 1
                 Scalene.__total_memory_free_samples += before - after
+            # If we just increased the peak memory, add to a leak score (UI to come)
+            if prevmax != Scalene.__max_footprint:
+                Scalene.__leak_score[fname][lineno] += (
+                    Scalene.__max_footprint - prevmax
+                )
 
     @staticmethod
     def memcpy_event_signal_handler(
@@ -1173,22 +1190,15 @@ class Scalene:
         """Print at most one line of the profile (true == printed one)."""
         if not Scalene.profile_this_code(fname, line_no):
             return False
+        # Currently disabled: surfacing the "leak score"
+        if Scalene.__leak_score[fname][line_no] > 0:
+            # print(fname,line_no,Scalene.__leak_score[fname][line_no] / Scalene.__total_memory_malloc_samples, Scalene.__leak_score[fname][line_no] / Scalene.__max_footprint)
+            pass
         current_max = Scalene.__max_footprint
         did_sample_memory: bool = (
             Scalene.__total_memory_free_samples
             + Scalene.__total_memory_malloc_samples
         ) > 0
-        # Strip newline
-        line = line.rstrip()
-        # Generate syntax highlighted version.
-        if Scalene.__html:
-            syntax_highlighted = Syntax(
-                line, "python", theme="default", line_numbers=False
-            )
-        else:
-            syntax_highlighted = Syntax(
-                line, "python", theme="vim", line_numbers=False
-            )
         # Prepare output values.
         n_cpu_samples_c = Scalene.__cpu_samples_c[fname][line_no]
         # Correct for negative CPU sample counts. This can happen
@@ -1224,18 +1234,36 @@ class Scalene:
             frees = Scalene.__memory_free_samples[fname][line_no][index]
             n_free_mb += frees
 
-        n_growth_mb = n_malloc_mb - n_free_mb
-        if -1 < n_growth_mb < 0:
-            # Don't print out "-0".
-            n_growth_mb = 0
         n_usage_fraction = (
             0
             if not Scalene.__total_memory_malloc_samples
             else n_malloc_mb / Scalene.__total_memory_malloc_samples
         )
         n_python_fraction = (
-            0 if not n_malloc_mb else n_python_malloc_mb / Scalene.__total_memory_malloc_samples # was / n_malloc_mb
+            0
+            if not n_malloc_mb
+            else n_python_malloc_mb
+            / Scalene.__total_memory_malloc_samples  # was / n_malloc_mb
         )
+
+        if False:
+            # Currently disabled; possibly use in another column?
+            # Correct for number of samples
+            for bytei in Scalene.__memory_malloc_count[fname][line_no]:
+                n_malloc_mb /= Scalene.__memory_malloc_count[fname][line_no][
+                    bytei
+                ]
+                n_python_malloc_mb /= Scalene.__memory_malloc_count[fname][
+                    line_no
+                ][bytei]
+            for bytei in Scalene.__memory_free_count[fname][line_no]:
+                n_free_mb /= Scalene.__memory_free_count[fname][line_no][bytei]
+
+        n_growth_mb = n_malloc_mb - n_free_mb
+        if -1 < n_growth_mb < 0:
+            # Don't print out "-0".
+            n_growth_mb = 0
+
         # Finally, print results.
         n_cpu_percent_c_str: str = (
             "" if n_cpu_percent_c < 0.5 else "%6.0f%%" % n_cpu_percent_c
@@ -1322,7 +1350,7 @@ class Scalene:
                     n_growth_mb_str,
                     nufs,  # spark_str + n_usage_fraction_str,
                     n_copy_mb_s_str,
-                    syntax_highlighted,
+                    line,
                 )
                 return True
             else:
@@ -1346,7 +1374,7 @@ class Scalene:
                     ncpps,  # n_cpu_percent_python_str,
                     ncpcs,  # n_cpu_percent_c_str,
                     sys_str,
-                    syntax_highlighted,
+                    line,
                 )
                 return True
             else:
@@ -1471,17 +1499,29 @@ class Scalene:
                 _, _, spark_str = sparkline.generate(
                     samples.get()[0 : samples.len()], 0, current_max
                 )
-                mem_usage_line = Text.assemble(
-                    "Memory usage: ",
-                    ((spark_str, "blue")),
-                    (" (max: %6.2fMB)\n" % current_max),
-                )
+                # If memory used is > 1GB, use GB as the unit.
+                if current_max > 1024:
+                    mem_usage_line = Text.assemble(
+                        "Memory usage: ",
+                        ((spark_str, "blue")),
+                        (" (max: %6.2fGB)\n" % (current_max / 1024)),
+                    )
+                else:
+                    # Otherwise, use MB.
+                    mem_usage_line = Text.assemble(
+                        "Memory usage: ",
+                        ((spark_str, "blue")),
+                        (" (max: %6.2fMB)\n" % current_max),
+                    )
                 title.append(mem_usage_line)
 
         null = open("/dev/null", "w")
         # Get column width of the terminal and adjust to fit.
         # Note that Scalene works best with at least 132 columns.
-        column_width = shutil.get_terminal_size().columns
+        if Scalene.__html:
+            column_width = 132
+        else:
+            column_width = shutil.get_terminal_size().columns
         console = Console(
             width=column_width,
             record=True,
@@ -1537,6 +1577,7 @@ class Scalene:
                 box=box.MINIMAL_HEAVY_HEAD,
                 title=new_title,
                 collapse_padding=True,
+                width=column_width - 1,
             )
 
             tbl.add_column("Line", justify="right", no_wrap=True)
@@ -1544,21 +1585,64 @@ class Scalene:
             tbl.add_column("Time %\nnative", no_wrap=True)
             tbl.add_column("Sys\n%", no_wrap=True)
 
+            other_columns_width = 0  # Size taken up by all columns BUT code
+
             if did_sample_memory:
                 tbl.add_column("Mem %\nPython", no_wrap=True)
                 tbl.add_column("Net\n(MB)", no_wrap=True)
                 tbl.add_column("Memory usage\nover time / %", no_wrap=True)
                 tbl.add_column("Copy\n(MB/s)", no_wrap=True)
-                tbl.add_column("\n" + fname, width=column_width - 72)
+                other_columns_width = 72
+                tbl.add_column(
+                    "\n" + fname,
+                    width=column_width - other_columns_width,
+                    no_wrap=True,
+                )
             else:
-                tbl.add_column("\n" + fname, width=column_width - 36)
+                other_columns_width = 36
+                tbl.add_column(
+                    "\n" + fname,
+                    width=column_width - other_columns_width,
+                    no_wrap=True,
+                )
 
             # Print out the the profile for the source, line by line.
             with open(fname, "r") as source_file:
                 # We track whether we should put in ellipsis (for reduced profiles)
                 # or not.
                 did_print = True  # did we print a profile line last time?
-                for line_no, line in enumerate(source_file, 1):
+                code_lines = source_file.read()
+                # Generate syntax highlighted version for the whole file,
+                # which we will consume a line at a time.
+                # See https://github.com/willmcgugan/rich/discussions/965#discussioncomment-314233
+                syntax_highlighted = None
+                if Scalene.__html:
+                    syntax_highlighted = Syntax(
+                        code_lines,
+                        "python",
+                        theme="default",
+                        line_numbers=False,
+                        code_width=None,
+                    )
+                else:
+                    syntax_highlighted = Syntax(
+                        code_lines,
+                        "python",
+                        theme="vim",
+                        line_numbers=False,
+                        code_width=None,
+                    )
+                capture_console = Console(
+                    width=column_width - other_columns_width,
+                    force_terminal=True,
+                )
+                formatted_lines = [
+                    SyntaxLine(segments)
+                    for segments in capture_console.render_lines(
+                        syntax_highlighted
+                    )
+                ]
+                for line_no, line in enumerate(formatted_lines, start=1):
                     old_did_print = did_print
                     did_print = Scalene.output_profile_line(
                         fname, LineNumber(line_no), line, console, tbl
@@ -1572,12 +1656,12 @@ class Scalene:
 
         if Scalene.__html:
             # Write HTML file.
-            if not Scalene.__output_file:
-                Scalene.__output_file = "/dev/stdout"
             md = Markdown(
                 "generated by the [scalene](https://github.com/emeryberger/scalene) profiler"
             )
             console.print(md)
+            if not Scalene.__output_file:
+                Scalene.__output_file = "/dev/stdout"
             console.save_html(Scalene.__output_file, clear=False)
         else:
             if not Scalene.__output_file:
